     /**
      * \brief Flat File Parser : Date Field Validator
      *
      * The data will be checked for a correct date in the configured format.
      *
      * <br><br>
      *
      * All native supported date formats can be used with this validator
      * (see ILE RPG Language Reference for a list of supported date formats).
      *
      * \author Mihael Schmidt
      * \date   12.04.2010
      *
      * \implements FFPVAL - Flat File Parser Field Validator
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain


      *---------------------------------------------------------------
      * Prototypes
      *---------------------------------------------------------------
      /copy 'date_validator_h.rpgle'
      /copy 'logger_h.rpgle'
      /copy 'libc_h.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D FIELD_VALIDATOR_ID...
     D                 C                   'ffp_validator_date'


      *---------------------------------------------------------------
      * Variables
      *---------------------------------------------------------------
      /copy 'validator_t.rpgle'


      *---------------------------------------------------------------
      * Procedures
      *---------------------------------------------------------------

     /**
      * \brief Create date field validator
      *
      * Creates an instance of the date field validator. The date
      * format is passed in the user data. If no user data is passed
      * *ISO will be used for validation.
      *
      * \param User data, null-terminated
      */
     P ffp_validator_date_create...
     P                 B                   export
     D                 PI              *
     D   userdata                      *   const options(*nopass)
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
      /free
       validator = %alloc(%size(ffp_validator));

       clear header;

       header.id = FIELD_VALIDATOR_ID;
       header.proc_validate = %paddr('ffp_validator_date_validate');
       header.proc_finalize = %paddr('ffp_validator_date_finalize');
       header.proc_setLogger = %paddr('ffp_validator_date_setLogger');

       if (%parms() = 0 or userdata = *null);
         header.userdata = *null;
       else;
         header.userdata = %alloc(strlen(userdata) + 1); // speicher fr null reservieren
         memcpy(header.userdata : userdata : strlen(userdata) + 1); // null mit kopieren
       endif;

       return validator;
      /end-free
     P                 E


     /**
      * \brief Dispose field validator
      *
      * Frees all allocated memory.
      *
      * \param Field validator
      */
     P ffp_validator_date_finalize...
     P                 B                   export
     D                 PI
     D   validator                     *
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
      /free
       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) validator;
      /end-free
     P                 E


     /**
      * \brief Validate field data
      *
      * Checks if the given field data is a valid date in the configured format.
      *
      * \param Field validator
      * \param Data
      */
     P ffp_validator_date_validate...
     P                 B                   export
     D                 PI              N
     D  validator                      *   const
     D  data                      65535A   const varying
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
     D l_format        S             10A
     D retVal          S               N   inz(*on)
     D l_data10        S             10A
     D l_data8         S              8A
     D l_data7         S              7A
     D l_data6         S              6A
     D l_data5         S              5A
     D l_length        S             10I 0
     D lengthCheckReturnCode...
     D                 S             10I 0
      *
     D format10        DS            40    qualified
     D   e1                          10A   overlay(format10 : *next) inz('*EUR')
     D   e2                          10A   overlay(format10 : *next) inz('*USA')
     D   e3                          10A   overlay(format10 : *next) inz('*ISO')
     D   e4                          10A   overlay(format10 : *next) inz('*JIS')
     D   entry                       10A   overlay(format10) dim(4)
      *
     D format8         DS            80    qualified
     D   e1                          10A   overlay(format8 : *next) inz('*EUR0')
     D   e2                          10A   overlay(format8 : *next) inz('*ISO0')
     D   e3                          10A   overlay(format8 : *next) inz('*USA0')
     D   e4                          10A   overlay(format8 : *next) inz('*JIS0')
     D   e5                          10A   overlay(format8 : *next) inz('*DMY')
     D   e6                          10A   overlay(format8 : *next) inz('*MDY')
     D   e7                          10A   overlay(format8 : *next) inz('*YMD')
     D   e8                          10A   overlay(format8  :*next)
     D                                     inz('*LONGJUL')
     D   entry                       10A   overlay(format8) dim(8)
      *
     D format7         DS            10    qualified
     D   e1                          10A   overlay(format7) inz('*LONGJUL0')
     D   entry                       10A   overlay(format7) dim(1)
      *
     D format6         DS            40    qualified
     D   e1                          10A   overlay(format6 : *next) inz('*DMY0')
     D   e2                          10A   overlay(format6 : *next) inz('*MDY0')
     D   e3                          10A   overlay(format6 : *next) inz('*YMD0')
     D   e4                          10A   overlay(format6 : *next) inz('*JUL')
     D   entry                       10A   overlay(format6) dim(4)
      *
     D format5         DS            10    qualified
     D   e1                          10A   overlay(format5) inz('*JUL0')
     D   entry                       10A   overlay(format5) dim(1)

      /free
       l_format = %str(header.userdata);
       l_length = %len(%trimr(data));

       // check length
       select;

         when (l_length = 10);
           lengthCheckReturnCode = %lookup(l_format : format10.entry);

         when (l_length = 8);
           lengthCheckReturnCode = %lookup(l_format : format8.entry);

         when (l_length = 7);
           lengthCheckReturnCode = %lookup(l_format : format7.entry);

         when (l_length = 6);
           lengthCheckReturnCode = %lookup(l_format : format6.entry);

         when (l_length = 5);
           lengthCheckReturnCode = %lookup(l_format : format5.entry);

         other;
           lengthCheckReturnCode = 0;

       endsl;


       if (lengthCheckReturnCode = 0);
         return *off;
       endif;


       // check format
       select;

         when (l_format = '*DMY');
           l_data8 = data;
           test(de) *DMY l_data8;

         when (l_format = '*DMY0');
           l_data6 = data;
           test(de) *DMY0 l_data6;

         when (l_format = '*ISO');
           l_data10 = data;
           test(de) *ISO l_data10;

         when (l_format = '*ISO0');
           l_data8 = data;
           test(de) *ISO0 l_data8;

         when (l_format = '*YMD');
           l_data8 = data;
           test(de) *YMD l_data8;

         when (l_format = '*YMD0');
           l_data6 = data;
           test(de) *YMD0 l_data6;

         when (l_format = '*EUR');
           l_data10 = data;
           test(de) *EUR l_data10;

         when (l_format = '*EUR0');
           l_data8 = data;
           test(de) *EUR0 l_data8;

         when (l_format = '*USA');
           l_data10 = data;
           test(de) *USA l_data10;

         when (l_format = '*USA0');
           l_data8 = data;
           test(de) *USA0 l_data8;

         when (l_format = '*LONGJUL');
           l_data8 = data;
           test(de) *LONGJUL l_data8;

         when (l_format = '*LONGJUL0');
           l_data7 = data;
           test(de) *LONGJUL0 l_data7;

         when (l_format = '*JUL');
           l_data6 = data;
           test(de) *JUL l_data6;

         when (l_format = '*JUL0');
           l_data5 = data;
           test(de) *JUL0 l_data5;

         when (l_format = '*MDY');
           l_data8 = data;
           test(de) *MDY l_data8;

         when (l_format = '*MDY0');
           l_data6 = data;
           test(de) *MDY0 l_data6;

         when (l_format = '*JIS');
           l_data10 = data;
           test(de) *JIS l_data10;

         when (l_format = '*JIS0');
           l_data8 = data;
           test(de) *JIS0 l_data8;

         other;
           return *off;

       endsl;

       if (%error());
         retVal = *off;

         ffp_logger_log(header.logger :
                        '[' + %trimr(header.id) + '] ' +
                        %trimr(data) + ' ist kein gltiges Datum im Format ' +
                        %trimr(l_format) + '.' : data);
       endif;

       return retVal;
      /end-free
     P                 E


     /**
      * \brief Set logger
      *
      * \param Validator
      * \param Loggr
      */
     P ffp_validator_date_setLogger...
     P                 B                   export
     D                 PI
     D   validator                     *   const
     D   logger                        *   const
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
      /free
       header.logger = logger;
      /end-free
     P                 E
