     /**
      * \brief Flat File Parser : Echo Line Processor
      *
      * Every data line will be given out raw with the number of
      * characters prepending and the line format name.
      *
      * \author Mihael Schmidt
      * \date   09.04.2010
      *
      * \implements FFPPRC - Flat File Parser Line Processor
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain


      *---------------------------------------------------------------
      * Prototypes
      *---------------------------------------------------------------
      /copy 'parser_h.rpgle'
      /copy 'echo_line_processor_h.rpgle'
      /copy 'libc_h.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D LINE_PROCESSOR_ID...
     D                 C                   'ffp_processor_echo'


      *---------------------------------------------------------------
      * Variables
      *---------------------------------------------------------------
      /copy 'line_processor_t.rpgle'


      *---------------------------------------------------------------
      * Procedures
      *---------------------------------------------------------------

     /**
      * \brief Create echo line processor
      *
      * Creates an instance of the echo line processor. No user data is needed.
      *
      * \param User data
      */
     P ffp_processor_echo_create...
     P                 B                   export
     D                 PI              *
     D   parser                        *   const 
     D   userdata                      *   const options(*nopass)
      *
     D header          DS                  likeds(ffp_processor)
     D                                     based(lineProcessor)
      /free
       lineProcessor = %alloc(%size(ffp_processor));

       clear header;

       header.id = LINE_PROCESSOR_ID;
       header.logger = *null;
       header.serializer = *null;
       header.proc_process = %paddr('ffp_processor_echo_process');
       header.proc_finalize = %paddr('ffp_processor_echo_finalize');
       header.proc_setLogger =
           %paddr('ffp_processor_echo_setLogger');
       header.proc_setSerializer =
           %paddr('ffp_processor_echo_setSerializer');

       if (%parms() = 0 or userdata = *null);
         header.userdata = *null;
       else;
         header.userdata = %alloc(strlen(userdata) + 1); // speicher fr null reservieren
         memcpy(header.userdata : userData : strlen(userdata) + 1); // null mit kopieren
       endif;

       return lineProcessor;
      /end-free
     P                 E


     /**
      * \brief Dispose line processor
      *
      * Frees all allocated memory.
      *
      * \param Line processor
      */
     P ffp_processor_echo_finalize...
     P                 B                   export
     D                 PI
     D   lineProcessor...
     D                                 *
      *
     D header          DS                  likeds(ffp_processor)
     D                                     based(lineProcessor)
      /free
       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) lineProcessor;
      /end-free
     P                 E


     /**
      * \brief Process line data
      *
      * Outputs the given data to the console.
      *
      * \param Line processor
      * \parma Parser
      * \param Event
      * \param Line format
      * \param Data
      */
     P ffp_processor_echo_process...
     P                 B                   export
     D                 PI
     D  lineProcessor                  *   const
     D  parser                         *   const
     D  event                        10I 0 const
     D  lineFormat                     *   const
     D  data                      65535A   const varying
     D  valid                          N   const
      *
     D header          DS                  likeds(ffp_processor)
     D                                     based(lineProcessor)
     D l_lineFormat    DS                  likeds(ffp_lineFormat)
     D                                     based(lineFormat)
     D l_data          S             40A
     D length          S             10I 0
     D dsp             S             52A
      /free
       if (event = DOCUMENT_START);
         dsply 'Document start ...';

       elseif (event = DOCUMENT_END);
         dsply 'Document end ...';

       elseif (event = LINE);
         length = %len(data);
         l_data = data;
         dsp = %trimr(%char(length) + ' ' + l_lineFormat.name + ': ' + l_data);
         dsply dsp;
       endif;
      /end-free
     P                 E


     /**
      * \brief Set logger
      *
      * \param Processor
      * \param Logger
      */
     P ffp_processor_echo_setLogger...
     P                 B                   export
     D                 PI
     D   processor                     *   const
     D   logger                        *   const
      *
     D header          DS                  likeds(ffp_processor)
     D                                     based(processor)
      /free
       header.logger = logger;
      /end-free
     P                 E


     /**
      * \brief Set serializer
      *
      * \param Processor
      * \param Serializer
      */
     P ffp_processor_echo_setSerializer...
     P                 B                   export
     D                 PI
     D   processor                     *   const
     D   serializer                    *   const
      *
     D header          DS                  likeds(ffp_processor)
     D                                     based(processor)
      /free
       header.serializer = serializer;
      /end-free
     P                 E

