     /**
      * \brief Flat File Parser : Logger Interface
      *
      * \author Mihael Schmidt
      * \date   05.05.2010
      *
      * \interface
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------

     H nomain

      /define INTERFACE


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D ffp_logger_check...
     D                 PR
     D   logger                        *   const
      *
      /copy 'reflection_h.rpgle'
      /copy 'logger_h.rpgle'
      /copy 'message_h.rpgle'


      *-------------------------------------------------------------------------
      * Templates
      *-------------------------------------------------------------------------
      /copy 'logger_t.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create logger
      *
      * Creates the logger.
      *
      * \param Service program (logger)
      * \param User data (optional)
      * \param Create procedure name (optional)
      *
      * \return Pointer to logger or *null if given service program does not
      *         fully implement the interface error handler.
      */
     P ffp_logger_create...
     P                 B                   export
     D                 PI              *
     D   serviceProgram...
     D                               10A   const
     D   userdata                      *   const options(*nopass)
     D   procedureName...
     D                              256A   const options(*nopass)
      *
     D createProcedureName...
     D                 S            256A
     D procedurePointer...
     D                 S               *   procptr
     D create          PR              *   extproc(procedurePointer)
     D    userdata                     *   const options(*nopass)
      *
     D initProcedureName...
     D                 S            256A
      *
     D loggerHeader...
     D                 DS                  likeds(ffp_logger) based(logger)
      /free
       if (%parms() = 3);
         createProcedureName = procedureName;
       else;
         createProcedureName = 'ffp_logger_create';
       endif;

       procedurePointer = reflection_getProcedurePointer(
                                  serviceProgram :
                                  %trimr(createProcedureName));

       if (procedurePointer <> *null);

           // input provider aufbauen/erstellen
           if (%parms() = 1);
             logger = create();
           else;
             logger = create(userData);
           endif;

       endif;

       return logger;
      /end-free
     P                 E


     /**
      * \brief Dispose logger
      *
      * Calls clean up process and frees allocated memory.
      *
      * \param Logger
      */
     P ffp_logger_finalize...
     P                 B                   export
     D                 PI
     D   logger                        *
      *
     D loggerHeader...
     D                 DS                  likeds(ffp_logger) based(logger)
     D procedurePointer...
     D                 S               *   procptr
     D finalize        PR                  extproc(procedurePointer)
     D   logger                        *   const
      /free
       procedurePointer = loggerHeader.proc_finalize;

       ffp_logger_check(logger);

       finalize(logger);
       dealloc(n) logger;
      /end-free
     P                 E


     /**
      * \brief Log message
      *
      * Logs the passed message.
      *
      * \param Logger
      * \param Message
      * \param Data
      * \param Message type
      *
      * \info No escape message is sent and no message logged if the logger is *null.
      */
     P ffp_logger_log...
     P                 B                   export
     D                 PI
     D   logger                        *   const
     D   message                  65535A   const varying
     D   data                     65535A   const varying options(*omit:*nopass)
     D   type                        10I 0 const options(*nopass)
      *
     D loggerHeader...
     D                 DS                  likeds(ffp_logger) based(logger)
      *
     D procedurePointer...
     D                 S               *   procptr
     D log             PR              N   extproc(procedurePointer)
     D   logger                        *   const
     D   message                  65535A   const varying
     D   data                     65535A   const varying options(*omit:*nopass)
     D   type                        10I 0 const options(*nopass)
      /free
       if (logger <> *null);

         procedurePointer = loggerHeader.proc_log;

         ffp_logger_check(logger);

         if (%parms() = 2);
           log(logger : message);
         elseif (%parms() = 3 and %addr(data) = *null);
           log(logger : message : *omit);
         elseif (%parms() = 3);
           log(logger : message : data);
         elseif (%parms() = 4 and %addr(data) = *null);
           log(logger : message : *omit : type);
         elseif (%parms() = 4);
           log(logger : message : data : type);
         endif;

       endif;
      /end-free
     P                 E


     /**
      * \brief Check logger
      *
      * Checks if the logger provides procedure pointers for all
      * procedures it should implement.
      *
      * \param Logger
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_logger_check...
     P                 B
     D                 PI
     D   logger                        *   const
      *
     D header          DS                  likeds(ffp_logger) based(logger)
     D cachedLoggerId...
     D                 S                   like(ffp_logger.id) static
     D loggerChecked...
     D                 S               N   inz(*off) static
      /free
       if (header.id = cachedLoggerId and loggerChecked);
         return;
       endif;

       cachedLoggerId = header.id;

       if (header.proc_create = *null);
         msg_sendEscapeMessage('Flat File Parser Logger Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       elseif (header.proc_finalize = *null);
         msg_sendEscapeMessage('Flat File Parser Logger Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       elseif (header.proc_log = *null);
         msg_sendEscapeMessage('Flat File Parser Logger Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id) +  '.');
       endif;

       loggerChecked = *on;
      /end-free
     P                 E
