     /**
      * \brief Flat File Parser : Serializer Interface
      *
      * The serializer module stores data in an external container
      * (f. e. database file or text file).
      *
      * \author Mihael Schmidt
      * \date   21.04.2010
      *
      * \interface
      *
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain

      /define INTERFACE


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D ffp_serializer_check...
     D                 PR
     D   serializer                    *   const
      *
      /copy 'serializer_h.rpgle'
      /copy 'serializer_t.rpgle'
      /copy 'message_h.rpgle'
      /copy 'reflection_h.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create serializer
      *
      * Creates the serializer.
      *
      * \param Service program (Serializer)
      * \param User data (optional)
      * \param Create procedure (default: ffp_serializer_create)
      *
      * \return Serializer
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_serializer_create...
     P                 B                   export
     D                 PI              *
     D   serviceProgram...
     D                               10A   const
     D   userdata                      *   const options(*nopass)
     D   createProcedure...
     D                              256A   const options(*nopass)
      *
     D procedurePointer...
     D                 S               *   procptr
     D create          PR              *   extproc(procedurePointer)
     D    userdata                     *   const options(*nopass)
      *
     D procedureName...
     D                 S            256A
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(serializer)
      /free
       if (%parms() = 3);
         procedureName = createProcedure;
       else;
         procedureName = 'ffp_serializer_create';
       endif;

       procedurePointer = reflection_getProcedurePointer(
                                  serviceProgram :
                                  %trimr(procedureName));

       if (procedurePointer <> *null);

           // line processor aufbauen/erstellen
           if (%parms() = 1);
             serializer = create();
           else;
             serializer = create(userData);
           endif;

       endif;

       ffp_serializer_check(serializer);

       return serializer;
      /end-free
     P                 E


     /**
      * \brief Dispose serializer
      *
      * Calls clean up process and frees allocated memory.
      *
      * \param Serializer
      */
     P ffp_serializer_finalize...
     P                 B                   export
     D                 PI
     D  serializer                     *
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(serializer)
     D procedurePointer...
     D                 S               *   procptr
     D finalize        PR                  extproc(procedurePointer)
     D   serializer                    *
      /free
       procedurePointer = header.proc_finalize;

       ffp_serializer_check(serializer);

       finalize(serializer);

       if (serializer <> *null);
         dealloc(n) serializer;
       endif;
      /end-free
     P                 E


     /**
      * \brief Serialize data
      *
      * Process the passed data.
      *
      * \param Serializer
      * \param Data
      */
     P ffp_serializer_serialize...
     P                 B                   export
     D                 PI          1000A   varying
     D   serializer                    *   const
     D   data                          *   const
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(serializer)
      *
     D serialize       PR          1000A   varying
     D                                     extproc(header.proc_serialize)
     D   serializer                    *   const
     D   data                          *   const
      /free
       if (serializer <> *null);
         ffp_serializer_check(serializer);

         return serialize(serializer : data);
       endif;

       return *blank;
      /end-free
     P                 E


     /**
      * \brief Check line processor
      *
      * Checks if the line processor provides procedure pointers for all
      * procedures it should implement.
      *
      * \param Line processor
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_serializer_check...
     P                 B
     D                 PI
     D   processor                     *   const
      *
     D header          DS                  likeds(ffp_serializer)
     D                                     based(processor)
     D cachedProviderId...
     D                 S                   like(ffp_serializer.id) static
     D providerChecked...
     D                 S               N   inz(*off) static
      /free
       if (header.id = cachedProviderId and providerChecked);
         return;
       endif;

       cachedProviderId = header.id;

       if (header.proc_serialize = *null);
         msg_sendEscapeMessage('Flat File Parser Serializer Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id)+
             '.');
       elseif (header.proc_finalize = *null);
         msg_sendEscapeMessage('Flat File Parser Serializer Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id)+
             '.');
       endif;

       providerChecked = *on;
      /end-free
     P                 E

