     /**
      * \brief Flat File Parser : Field Validator Interface
      *
      *
      * \author Mihael Schmidt
      * \date   12.04.2010
      *
      * \interface
      *
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain

      /define INTERFACE


      *-------------------------------------------------------------------------
      * Prototypes
      *-------------------------------------------------------------------------
     D ffp_validator_check...
     D                 PR
     D   validator                     *   const
      *
      /copy 'validator_h.rpgle'
      /copy 'validator_t.rpgle'
      /copy 'message_h.rpgle'
      /copy 'reflection_h.rpgle'


      *-------------------------------------------------------------------------
      * Procedures
      *-------------------------------------------------------------------------


     /**
      * \brief Create field validator
      *
      * Creates a field validator.
      *
      * \param Service program (Field Validator)
      * \param User data , null-terminated (optional)
      * \param Create procedure (default: ffp_validator_create)
      *
      * \return Field Validator
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_validator_create...
     P                 B                   export
     D                 PI              *
     D   serviceProgram...
     D                               10A   const
     D   userdata                      *   const options(*nopass)
     D   createProcedure...
     D                              256A   const options(*nopass)
      *
     D procedurePointer...
     D                 S               *   procptr
     D create          PR              *   extproc(procedurePointer)
     D    userdata                     *   const options(*nopass)
      *
     D procedureName...
     D                 S            256A
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
      /free
       if (%parms() = 3);
         procedureName = createProcedure;
       else;
         procedureName = 'ffp_validator_create';
       endif;

       procedurePointer = reflection_getProcedurePointer(
                                  serviceProgram :
                                  %trimr(procedureName));

       if (procedurePointer <> *null);

         // field validator aufbauen/erstellen
         if (%parms() = 1);
           validator = create();
         else;
           validator = create(userData);
         endif;

       endif;

       ffp_validator_check(validator);

       return validator;
      /end-free
     P                 E


     /**
      * \brief Dispose field validator
      *
      * Calls clean up process and frees allocated memory.
      *
      * \param Field validator
      */
     P ffp_validator_finalize...
     P                 B                   export
     D                 PI
     D  validator                      *
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
     D procedurePointer...
     D                 S               *   procptr
     D finalize        PR                  extproc(procedurePointer)
     D   validator                     *
      /free
       procedurePointer = header.proc_finalize;

       ffp_validator_check(validator);

       finalize(validator);

       if (validator <> *null);
         dealloc(n) validator;
       endif;
      /end-free
     P                 E


     /**
      * \brief Validate field data
      *
      * Validate the field data.
      *
      * \param Field validator
      * \param Data
      */
     P ffp_validator_validate...
     P                 B                   export
     D                 PI              N
     D   validator                     *   const
     D   data                     65535A   const varying
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
      *
     D validate        PR              N   extproc(header.proc_validate)
     D   validator                     *   const
     D   data                     65535A   const varying
      /free
       ffp_validator_check(validator);

       return validate(validator : data);
      /end-free
     P                 E


     /**
      * \brief Set logger
      *
      * \param Validator
      * \param Logger
      */
     P ffp_validator_setLogger...
     P                 B                   export
     D                 PI
     D   validator                     *   const
     D   logger                        *   const
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
      *
     D setLogger...
     D                 PR                  extproc(header.proc_setLogger)
     D   validator                     *   const
     D   logger                        *   const
      /free
       ffp_validator_check(validator);

       setLogger(validator : logger);
      /end-free
     P                 E


     /**
      * \brief Check field validator
      *
      * Checks if the field validator provides procedure pointers for all
      * procedures it should implement.
      *
      * \param Field validator
      *
      * \throws CPF9898 Incomplete implementation
      */
     P ffp_validator_check...
     P                 B
     D                 PI
     D   validator                     *   const
      *
     D header          DS                  likeds(ffp_validator)
     D                                     based(validator)
     D cachedValidatorId...
     D                 S                   like(ffp_validator.id) static
     D validatorChecked...
     D                 S               N   inz(*off) static
      /free
       if (header.id = cachedValidatorId and validatorChecked);
         return;
       endif;

       cachedValidatorId = header.id;

       if (header.proc_validate = *null);
         msg_sendEscapeMessage('Flat File Parser Field Validator Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id)+
             '.');
       elseif (header.proc_finalize = *null);
         msg_sendEscapeMessage('Flat File Parser Field Validator Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id)+
             '.');
       elseif (header.proc_setLogger = *null);
         msg_sendEscapeMessage('Flat File Parser Field Validator Interface: ' +
             'Incomplete Implementation ' + %trimr(header.id)+
             '.');
       endif;

       validatorChecked = *on;
      /end-free
     P                 E


