     /**
      * \brief Flat File Parser : XML Configuration Loader
      *
      * This modules contains the procedures for loading a xml ffp configuration
      * file via the xml bifs and opcodes introduced in V5R4.
      *
      * \author Mihael Schmidt
      * \date   23.01.2008
      *
      * \implements FFPCFG - Flat File Parser Configuration Provider
      */

      *-------------------------------------------------------------------------
      *
      * (C) Copyleft 2011 Mihael Schmidt
      *
      * This file is part of Flat File Parser (FFP).
      *
      * FFP is free software: you can redistribute it and/or modify
      * it under the terms of the GNU General Public License as published
      * the Free Software Foundation, either version 3 of the License, or
      * any later version.
      *
      * FFP is distributed in the hope that it will be useful,
      * but WITHOUT ANY WARRANTY; without even the implied warranty of
      * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
      * GNU General Public License for more details.
      *
      * You should have received a copy of the GNU General Public License
      * along with FFP.  If not, see <http://www.gnu.org/licenses/>.
      *
      *-------------------------------------------------------------------------


     H nomain


      *---------------------------------------------------------------
      * Prototypes
      *---------------------------------------------------------------
     D xmlHandler      PR            10I 0
     D   commArea                          likeds(tmpl_xmldata)
     D   event                       10I 0 value
     D   string                        *   value
     D   stringLen                   20I 0 value
     D   exceptionId                 10I 0 value
      *
     D loadingPostAction...
     D                 PR
     D   parserPtr                     *   const

      /copy 'parser_h.rpgle'
      /copy 'xml_config_provider_h.rpgle'
      /copy 'logger_h.rpgle'
      /copy 'line_processor_h.rpgle'
      /copy 'serializer_h.rpgle'
      /copy 'validator_h.rpgle'
      /copy 'message_h.rpgle'
      /copy 'llist_h.rpgle'
      /copy 'libc_h.rpgle'
      /copy 'rng/input_provider_h.rpgle'


      *-------------------------------------------------------------------------
      * Constants
      *-------------------------------------------------------------------------
     D CONFIG_PROVIDER_ID...
     D                 C                   'ffp_config_xml'


      *---------------------------------------------------------------
      * Variables
      *---------------------------------------------------------------
     D tmpl_xmldata    DS                  qualified based(nullPointer)
     D   parserPtr                     *
     D   lineFormat                    *
     D   lineFormatName...
     D                              100A   varying
     D   lineFormatType...
     D                               10A
     D   lineFormatString...
     D                            10000A
     D   lastAttrName               100A
     D   lastElement                100A
     D   serviceProgram...
     D                               10A
     D   procedure                  256A
     D   parameter                 1000A
     D   fieldCount                  10I 0
      *
      /copy 'config_provider_t.rpgle'


     /**
      * \brief Create XML config provider
      *
      * The user data should contain the file name of the FFP XML
      * configuration file. The file name must be null terminated.
      *
      * \param User data
      */
     P ffp_config_xml_create...
     P                 B                   export
     D                 PI              *
     D   userdata                      *   const options(*nopass)
      *
     D header          DS                  likeds(ffp_config_provider)
     D                                     based(configProvider)
      /free
       configProvider = %alloc(%size(ffp_config_provider));

       clear header;

       header.id = CONFIG_PROVIDER_ID;
       header.proc_load = %paddr('ffp_config_xml_load');
       header.proc_finalize = %paddr('ffp_config_xml_finalize');

       if (%parms() = 0 or userdata = *null);
         header.userdata = *null;
       else;
         header.userdata = %alloc(strlen(userdata) + 1); // speicher fr null reservieren
         memcpy(header.userdata : userData : strlen(userdata) + 1); // null mit kopieren
       endif;

       return configProvider;
      /end-free
     P                 E


     /**
      * \brief Dispose config provider
      *
      * Frees all allocated memory.
      *
      * \param Config Provider
      */
     P ffp_config_xml_finalize...
     P                 B                   export
     D                 PI
     D   configProvider...
     D                                 *
      *
     D header          DS                  likeds(ffp_config_provider)
     D                                     based(configProvider)
      /free
       if (header.userdata <> *null);
         dealloc header.userdata;
       endif;

       dealloc(n) configProvider;
      /end-free
     P                 E


     /**
      * \brief Load configuration from XML file
      *
      * \author Mihael Schmidt
      * \date   23.01.2008
      *
      * \param Parser
      * \param XML configuration file path
      *
      * \throws XML Parse error
      */
     P ffp_config_xml_load...
     P                 B                   export
     D                 PI
     D   pConfigProvider...
     D                                 *   const
     D   parser                        *   const
      *
     D configProvider...
     D                 DS                  likeds(ffp_config_provider)
     D                                     based(pConfigProvider)
     D retVal          S               N   inz(*on)
     D xmlCommArea     DS                  likeds(tmpl_xmldata)
     D file            S           1024A
      /free
       file = %str(configProvider.userdata);

       xmlCommArea.parserPtr = parser;

       xml-sax %handler(xmlHandler : xmlCommArea)
               %xml(%trimr(file) : 'doc=file');

       loadingPostAction(parser);
      /end-free
     P                 E


     /**
      * \brief XML handler procedure for config loading
      *
      * \author Mihael Schmidt
      * \date   22.01.2008
      */
     P xmlHandler      B
     D                 PI            10I 0
     D   commArea                          likeds(tmpl_xmldata)
     D   event                       10I 0 value
     D   string                        *   value
     D   stringLen                   20I 0 value
     D   exceptionId                 10I 0 value
      *
     D value           S           1000A   based(string)
     D tmp             S            100A
     D retVal          S             10I 0
      *
     D fieldLength     S             10U 0
      /free
       select;
         when (event = *XML_START_ELEMENT);
           if (%subst(value : 1 : stringLen) = 'lineFormat');
             // clear fields
             commArea.lineFormat = *null;
             %len(commArea.lineFormatName) = 0;
             commArea.lineFormatType = *blank;
             commArea.lineFormatString = *blank;
             commArea.fieldCount = 0;
           endif;

           commArea.lastAttrName = *blank;
           commArea.lastElement = %subst(value : 1 : stringLen);

         when (event = *XML_ATTR_NAME);
           commArea.lastAttrName = %subst(value : 1 : stringLen);

         when (event = *XML_ATTR_CHARS);
           if (commArea.lastElement = 'lineFormat');
             if (commArea.lastAttrName = 'name');
               commArea.lineFormatName = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'type');
               commArea.lineFormatType = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'imageString');
               commArea.lineFormatString = %subst(value : 1 : stringLen)+x'00';
             endif;

           elseif (commArea.lastElement = 'field' and
                   commArea.lastAttrName = 'length');
             monitor;
               fieldLength = %uns(%subst(value : 1 : stringLen));
               // add the field to the last line format
               ffp_addLineFormatField(commArea.lineFormat : fieldLength);
               commArea.fieldCount += 1;
               on-error;
                 msg_sendProgramMessage('Skipping field for line format ' +
                                        %trimr(commArea.lineFormatString) +
                                        '.');
             endmon;


           elseif (commArea.lastElement = 'validator');
             if (commArea.lastAttrName = 'serviceprogram');
               commArea.serviceprogram = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'procedure');
               commArea.procedure = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'parameter');
               %str(%addr(commArea.parameter) :
                    %size(commArea.parameter)) =
                        %subst(value : 1 : stringLen);
             endif;


           elseif (commArea.lastElement = 'lineProcessor');
             if (commArea.lastAttrName = 'serviceprogram');
               commArea.serviceprogram =
                   %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'procedure');
               commArea.procedure = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'parameter');
               %str(%addr(commArea.parameter) :
                    %size(commArea.parameter)) =
                        %subst(value : 1 : stringLen);
             endif;


           elseif (commArea.lastElement = 'inputProvider');
             if (commArea.lastAttrName = 'serviceprogram');
               commArea.serviceprogram =
                   %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'procedure');
               commArea.procedure = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'parameter');
               %str(%addr(commArea.parameter) :
                    %size(commArea.parameter)) =
                        %subst(value : 1 : stringLen);
             endif;


           elseif (commArea.lastElement = 'logger');
             if (commArea.lastAttrName = 'serviceprogram');
               commArea.serviceprogram =
                   %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'procedure');
               commArea.procedure = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'parameter');
               %str(%addr(commArea.parameter) :
                    %size(commArea.parameter)) =
                        %subst(value : 1 : stringLen);
             endif;


           elseif (commArea.lastElement = 'serializer');
             if (commArea.lastAttrName = 'serviceprogram');
               commArea.serviceprogram =
                   %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'procedure');
               commArea.procedure = %subst(value : 1 : stringLen);
             elseif (commArea.lastAttrName = 'parameter');
               %str(%addr(commArea.parameter) :
                    %size(commArea.parameter)) =
                        %subst(value : 1 : stringLen);
             endif;


           elseif (commArea.lastElement = 'config');
             tmp = %subst(value : 1 : stringLen);

             if (commArea.lastAttrName = 'ignoreBlankLines');
               if (tmp = 'yes');
                 ffp_setIgnoreBlankLines(commArea.parserPtr : *on);
               elseif (tmp = 'no');
                 ffp_setIgnoreBlankLines(commArea.parserPtr : *off);
               endif;

             elseif (commArea.lastAttrName = 'processInvalidLines');
               if (tmp = 'yes');
                 ffp_setProcessInvalidLines(commArea.parserPtr : *on);
               elseif (tmp = 'no');
                 ffp_setProcessInvalidLines(commArea.parserPtr : *off);
               endif;

             endif;
           endif;

           // add the line format if all needed data has been collected
           if (commArea.lineFormat = *null and
               commArea.lineFormatType <> *blank and
               commArea.lineFormatString <> *blank);

             if (commArea.lineFormatType = 'var');
               commArea.lineFormat = ffp_addLineFormatFromImage(
                   commArea.parserPtr : commArea.lineFormatName :
                   %addr(commArea.lineFormatString));
             elseif (commArea.lineFormatType = 'const');
               commArea.lineFormat = ffp_addConstantLineFormat(
                   commArea.parserPtr : commArea.lineFormatName :
                   %addr(commArea.lineFormatString));
             endif;

           endif;


         when (event = *XML_END_ELEMENT);
           if (commArea.lastElement = 'validator');

             if (commArea.parameter = *blank);
               ffp_addValidator(commArea.lineFormat :
                                commArea.fieldCount :
                                ffp_validator_create(
                                    commArea.serviceprogram :
                                    *null :
                                    commArea.procedure));
             else;
               ffp_addValidator(commArea.lineFormat :
                                commArea.fieldCount :
                                ffp_validator_create(
                                    commArea.serviceprogram :
                                    %addr(commArea.parameter) :
                                    commArea.procedure));
             endif;

             commArea.serviceprogram = *blank;
             commArea.procedure = *blank;
             commArea.parameter = *blank;

             commArea.lastElement = 'field';

           elseif (commArea.lastElement = 'lineProcessor');
             if (commArea.parameter = *blank);
               ffp_addLineProcessor(commArea.parserPtr :
                                    ffp_processor_create(
                                        commArea.serviceprogram :
                                        commArea.parserPtr :
                                        *null :
                                        commArea.procedure));
             else;
               ffp_addLineProcessor(commArea.parserPtr :
                                    ffp_processor_create(
                                        commArea.serviceprogram :
                                        commArea.parserPtr :
                                        %addr(commArea.parameter) :
                                        commArea.procedure));
             endif;

             commArea.serviceprogram = *blank;
             commArea.procedure = *blank;
             commArea.parameter = *blank;

             commArea.lastElement = *blank;

           elseif (commArea.lastElement = 'inputProvider');
             if (commArea.parameter = *blank);
               ffp_setInputProvider(commArea.parserPtr :
                                    rng_input_create(
                                        commArea.serviceprogram :
                                        *null :
                                        commArea.procedure));
             else;
               ffp_setInputProvider(commArea.parserPtr :
                                    rng_input_create(
                                        commArea.serviceprogram :
                                        %addr(commArea.parameter) :
                                        commArea.procedure));
             endif;

             commArea.serviceprogram = *blank;
             commArea.procedure = *blank;
             commArea.parameter = *blank;

             commArea.lastElement = *blank;

           elseif (commArea.lastElement = 'logger');
             if (commArea.parameter = *blank);
               ffp_setLogger(commArea.parserPtr :
                             ffp_logger_create(
                                 commArea.serviceprogram :
                                 *null :
                                 commArea.procedure));
             else;
               ffp_setLogger(commArea.parserPtr :
                             ffp_logger_create(
                                 commArea.serviceprogram :
                                 %addr(commArea.parameter) :
                                 commArea.procedure));
             endif;

             commArea.serviceprogram = *blank;
             commArea.procedure = *blank;
             commArea.parameter = *blank;

             commArea.lastElement = *blank;

           elseif (commArea.lastElement = 'serializer');
             if (commArea.parameter = *blank);
               ffp_setSerializer(commArea.parserPtr :
                                 ffp_serializer_create(
                                        commArea.serviceprogram :
                                        *null :
                                        commArea.procedure));
             else;
               ffp_setSerializer(commArea.parserPtr :
                                 ffp_serializer_create(
                                        commArea.serviceprogram :
                                        %addr(commArea.parameter) :
                                        commArea.procedure));
             endif;

             commArea.serviceprogram = *blank;
             commArea.procedure = *blank;
             commArea.parameter = *blank;

             commArea.lastElement = *blank;
           endif;
       endsl;

       return retVal;
      /end-free
     P                 E


     /**
      * \brief Loading post action
      *
      * Finishing touches after loading the config.
      *
      * \param Parser
      */
     P loadingPostAction...
     P                 B
     D                 PI
     D   parser                        *   const
      *
     D lineFormats     S               *
     D lineFormat      DS                  likeds(ffp_lineFormat)
     D                                     based(lineFormatPtr)
     D lineProcessors  S               *
     D lineProcessor   S               *   based(lineProcessorPtr)
     D i               S             10I 0
      /free
       lineFormats = ffp_getLineFormats(parser);

       // set logger on validators
       lineFormatPtr = list_getNext(lineFormats);
       dow (lineFormatPtr <> *null);

         for i = 1 to lineFormat.validatorCount;
           ffp_validator_setLogger(
               lineFormat.validators(i) :
               ffp_getLogger(parser));
         endfor;

         lineFormatPtr = list_getNext(lineFormats);
       enddo;

       // set logger and serializer on line processors
       lineProcessors = ffp_getLineProcessors(parser);
       list_abortIteration(lineProcessors);
       lineProcessorPtr = list_getNext(lineProcessors);
       dow (lineProcessorPtr <> *null);
         ffp_processor_setLogger(lineProcessor :
                                 ffp_getLogger(parser));
         ffp_processor_setSerializer(lineProcessor :
                                     ffp_getSerializer(parser));
         lineProcessorPtr = list_getNext(lineProcessors);
       enddo;
      /end-free
     P                 E
